@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -15,8 +15,8 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/core/application.h>
-#include <mobius/database/database.h>
+#include "application.h"
+#include <mobius/database/connection_pool.h>
 #include <mobius/exception.inc>
 #include <mobius/exception_posix.inc>
 #include <mobius/uri.h>
@@ -29,53 +29,8 @@ namespace mobius
 {
 namespace core
 {
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief application data structure
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static struct
+namespace
 {
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // constants
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  const std::string id = "mobiusft";
-  const std::string name = "Mobius Forensic Toolkit";
-  const std::string version = "0.5.30";
-  const std::string title = "Mobius v0.5.30";
-  const std::string copyright = "Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017 Eduardo Aguiar";
-  const std::string data_folder = "/home/aguiar/tmp/mobius.install/share/" + id;
-  const std::string os_name = "x86_64-suse-linux-gnu";
-  
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  // variables
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  bool is_loaded = false;
-  std::string config_folder;
-  std::string cache_folder;
-  mobius::database::database config_db;
-} data_;
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief join a relative path to a absolute path
-//! \param abs_path absolute path
-//! \param rel_path relative path
-//! \return absolute path
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static const std::string
-join_path (const std::string& abs_path, const std::string& rel_path)
-{
-  mobius::uri url;
-  url.set_scheme ("file");
-  url.set_path (abs_path);
-  url.join_path (rel_path);
-
-  std::string fullpath = get_path_from_url (url.get_value ());
-
-  if (fullpath.substr (0, abs_path.length ()) != abs_path)
-    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("invalid path"));
-
-  return fullpath;
-}
-
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief create directory recursively
 //! \param path directory path
@@ -106,56 +61,154 @@ make_directory (const std::string& path)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create application object
+//! \brief join a relative path to a absolute path
+//! \param abs_path absolute path
+//! \param rel_path relative path
+//! \return absolute path
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static const std::string
+join_path (const std::string& abs_path, const std::string& rel_path)
+{
+  mobius::uri url;
+  url.set_scheme ("file");
+  url.set_path (abs_path);
+  url.join_path (rel_path);
+
+  std::string fullpath = get_path_from_url (url.get_value ());
+
+  if (fullpath.substr (0, abs_path.length ()) != abs_path)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("invalid path"));
+
+  return fullpath;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief application data structure
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static struct
+{
+  //! \brief application ID
+  const std::string id = "mobiusft";
+
+  //! \brief application name
+  const std::string name = "Mobius Forensic Toolkit";
+
+  //! \brief application version
+  const std::string version = "1.5";
+
+  //! \brief application title
+  const std::string title = "Mobius v1.5";
+
+  //! \brief application copyright
+  const std::string copyright = "Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar";
+
+  //! \brief application OS name  
+  const std::string os_name = "x86_64-unknown-linux-gnu";
+
+  //! \brief data folder path
+  const std::string data_folder = "/home/aguiar/tmp/mobius.install/share/" + id;
+  
+  //! \brief configuration folder path
+  std::string config_folder;
+
+  //! \brief cache folder path
+  std::string cache_folder;
+
+  //! \brief module's connection pool
+  mobius::database::connection_pool pool;
+  
+  //! \brief flag object loaded
+  bool is_loaded = false;
+
+} data_;
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief initialize data structure
 //! \see https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-application::application ()
+static void
+load_data ()
 {
-  if (!data_.is_loaded)
-    {
-      const char *home = getenv ("HOME");
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // check if it is already loaded
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  if (data_.is_loaded)
+    return;
+
+  const char *home = getenv ("HOME");
       
-      // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-      // config folder
-      // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-      const char *xdg_config_home = getenv ("XDG_CONFIG_HOME");
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // config folder
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  const char *xdg_config_home = getenv ("XDG_CONFIG_HOME");
       
-      if (xdg_config_home)
-        data_.config_folder = xdg_config_home;
+  if (xdg_config_home)
+    data_.config_folder = xdg_config_home;
 
-      else
-        data_.config_folder = home + std::string ("/.config");
+  else
+    data_.config_folder = home + std::string ("/.config");
 
-      data_.config_folder += std::string ("/") + data_.id;
+  data_.config_folder += std::string ("/") + data_.id;
 
-      make_directory (data_.config_folder);
+  make_directory (data_.config_folder);
       
-      // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-      // cache folder
-      // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-      const char *xdg_cache_home = getenv ("XDG_CACHE_HOME");
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // cache folder
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  const char *xdg_cache_home = getenv ("XDG_CACHE_HOME");
       
-      if (xdg_cache_home)
-        data_.cache_folder = xdg_cache_home;
+  if (xdg_cache_home)
+    data_.cache_folder = xdg_cache_home;
 
-      else
-        data_.cache_folder = home + std::string ("/.cache");
+  else
+    data_.cache_folder = home + std::string ("/.cache");
 
-      data_.cache_folder += std::string ("/") + data_.id;
+  data_.cache_folder += std::string ("/") + data_.id;
 
-      make_directory (data_.cache_folder);
+  make_directory (data_.cache_folder);
       
-      // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-      // set loaded
-      // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-      data_.is_loaded = true;
-    }
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // database
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  const std::string db_path = join_path (data_.config_folder, data_.id + ".sqlite");
+  data_.pool.set_path (db_path);
+
+  auto db = data_.pool.get_database ();
+  auto transaction = db.new_transaction ();
+
+  db.execute ("PRAGMA foreign_keys = ON;");
+
+  db.execute (
+      "CREATE TABLE IF NOT EXISTS config"
+                            "(var TEXT PRIMARY KEY,"
+                           "value TEXT NULL);"
+         );
+  
+  transaction.commit ();
+
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // set data loaded
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  data_.is_loaded = true;
+}
+
+} // namespace
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get database object for current thread
+//! \return database object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::database::database
+get_database ()
+{
+  load_data ();
+  return data_.pool.get_database ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get application ID
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_id () const
 {
   return data_.id;
@@ -164,7 +217,7 @@ application::get_id () const
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get application name
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_name () const
 {
   return data_.name;
@@ -173,7 +226,7 @@ application::get_name () const
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get application version
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_version () const
 {
   return data_.version;
@@ -182,7 +235,7 @@ application::get_version () const
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get application title
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_title () const
 {
   return data_.title;
@@ -191,7 +244,7 @@ application::get_title () const
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get application copyright
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_copyright () const
 {
   return data_.copyright;
@@ -200,20 +253,32 @@ application::get_copyright () const
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get application host OS
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_os_name () const
 {
   return data_.os_name;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get data path
+//! \param path relative path
+//! \return absolute path to data_folder sub-path
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+application::get_data_path (const std::string& path) const
+{
+  return join_path (data_.data_folder, path);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get config path
 //! \param path relative path
 //! \return absolute path to config_folder sub-path
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_config_path (const std::string& path) const
 {
+  load_data ();
   return join_path (data_.config_folder, path);
 }
 
@@ -222,41 +287,32 @@ application::get_config_path (const std::string& path) const
 //! \param path relative path
 //! \return absolute path to cache_folder sub-path
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_cache_path (const std::string& path) const
 {
+  load_data ();
   return join_path (data_.cache_folder, path);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get data path
-//! \param path relative path
-//! \return absolute path to data_folder sub-path
+//! \brief create new connection to application database
+//! \return new connection object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
-application::get_data_path (const std::string& path) const
+mobius::database::connection_pool::connection
+application::new_connection ()
 {
-  return join_path (data_.data_folder, path);
+  return data_.pool.acquire ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create config database
+//! \brief create new transaction to application database
+//! \return new database transaction
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const mobius::database::database
-application::get_config_database () const
+mobius::database::transaction
+application::new_transaction ()
 {
-  if (!data_.config_db)
-    {
-      data_.config_db = mobius::database::database (get_config_path (data_.id + ".sqlite"));
-
-      data_.config_db.exec (
-          "CREATE TABLE IF NOT EXISTS config"
-                "(var TEXT PRIMARY KEY,"
-                 "value TEXT NULL);"
-         );
-    }
-
-  return data_.config_db;
+  auto db = get_database ();
+  return db.new_transaction ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -267,7 +323,7 @@ application::get_config_database () const
 bool
 application::has_config (const std::string& var) const
 {
-  auto db = get_config_database ();
+  auto db = get_database ();
   auto statement = db.new_statement ("SELECT value FROM config WHERE var = ?");
   statement.bind (1, var);
 
@@ -282,7 +338,7 @@ application::has_config (const std::string& var) const
 int
 application::get_config_int (const std::string& var) const
 {
-  auto db = get_config_database ();
+  auto db = get_database ();
   auto statement = db.new_statement ("SELECT value FROM config WHERE var = ?");
   statement.bind (1, var);
   
@@ -302,7 +358,7 @@ application::get_config_int (const std::string& var) const
 double
 application::get_config_double (const std::string& var) const
 {
-  auto db = get_config_database ();
+  auto db = get_database ();
   auto statement = db.new_statement ("SELECT value FROM config WHERE var = ?");
   statement.bind (1, var);
   
@@ -319,10 +375,10 @@ application::get_config_double (const std::string& var) const
 //! \param var var name
 //! \return string var
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 application::get_config_string (const std::string& var) const
 {
-  auto db = get_config_database ();
+  auto db = get_database ();
   auto statement = db.new_statement ("SELECT value FROM config WHERE var = ?");
   statement.bind (1, var);
   
@@ -342,11 +398,11 @@ application::get_config_string (const std::string& var) const
 void
 application::set_config (const std::string& var, int value) const
 {
-  auto db = get_config_database ();
+  auto db = get_database ();
   auto statement = db.new_statement ("INSERT OR REPLACE INTO config VALUES (?, ?)");
   statement.bind (1, var);
   statement.bind (2, value);
-  statement.exec ();
+  statement.execute ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -357,11 +413,11 @@ application::set_config (const std::string& var, int value) const
 void
 application::set_config (const std::string& var, double value) const
 {
-  auto db = get_config_database ();
+  auto db = get_database ();
   auto statement = db.new_statement ("INSERT OR REPLACE INTO config VALUES (?, ?)");
   statement.bind (1, var);
   statement.bind (2, value);
-  statement.exec ();
+  statement.execute ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -372,11 +428,11 @@ application::set_config (const std::string& var, double value) const
 void
 application::set_config (const std::string& var, const std::string& value) const
 {
-  auto db = get_config_database ();
+  auto db = get_database ();
   auto statement = db.new_statement ("INSERT OR REPLACE INTO config VALUES (?, ?)");
   statement.bind (1, var);
   statement.bind (2, value);
-  statement.exec ();
+  statement.execute ();
 }
 
 } // namespace core
